/*
	parse.y ver1.42
	1996/12/9
	by Kanaoka,Kanayama,Kaneko
*/
%{
#include	<stdio.h>
#include	"parse.h"
%}

%union{
	char			*symbol;
	struct const_data	value;
	struct filtree		*filtr;
	struct dcltree		*dcltr;
	struct spctree		*spctr;
	struct expr		*expr;
	struct liner		*liner;
	char			op;
	struct stmtree1		*stmt1;
	struct stmtree2		*stmt2;
}

%token	<symbol>	IDENTIFIER
%token	<value>		CONSTANT
%token	<symbol>	STRINGS
%token	<symbol>	CHARACTER

%token			STRUCT UNION ENUM
%token			AUTO STATIC EXTERN REGISTER CONST VOLATILE TYPEDEF

%token			CHAR INT SHORT LONG  FLOAT DOUBLE VOID
%token			SIGNED UNSIGNED TYPEDEF_TYPE

%token			IF ELSE WHILE FOR DO SWITCH CASE DEFAULT
%token			RETURN GOTO BREAK CONTINUE SIZEOF

%token			';' '(' ')' '[' ']' '{' '}' ','
%token			'?' ':' '|' '^' '&' '*' '.'
%token			DDDOT

%token	<op>		addop mulop unop
%token			shiftop_R shiftop_L compop_GT compop_LT compop_GE compop_LE eqop neqop
%token			assignop assignop_ADD assignop_SUB assignop_MOD assignop_AND assignop_MUL
%token			assignop_DIV assignop_SHL assignop_SHR assignop_XOR assignop_OR
%token			inc dec logor logand pomem

%type	<filtr>		file extern_declaration func_def
%type	<dcltr>		declaration declaration_list type_name
%type	<dcltr>		init_declarator init_declarator_list
%type	<dcltr>		member_declaration member_declaration_list
%type	<dcltr>		member_declarator member_declarator_list
%type	<dcltr>		enumerator enumerator_list
%type	<dcltr>		declarator declarator2
%type	<dcltr>		abst_declarator abst_declarator2
%type	<dcltr>		ident_list
%type	<dcltr>		param_type_list param_list param_decl
%type	<dcltr>		initializer initializer_list

%type	<spctr>		declaration_spec storage_class_spec type_spec type_qualifier
%type	<spctr>		spec_qualifier_list
%type	<spctr>		type_qualifier_list pointer
%type	<spctr>		struct_type_spec union_type_spec enum_type_spec

%type	<expr>		nc_expression expression expression_list
%type	<liner>		linear_operators

%type	<stmt1>		statement_list
%type	<stmt1>		statement

%type	<stmt2>		simple_statement
%type	<stmt2>		compound_statement
%type	<stmt2>		conditional_statements
%type	<stmt2>		iteration_statements
%type	<stmt2>		selection_statements
%type	<stmt2>		selection_statements1
%type	<stmt2>		jump_statements
%type	<stmt2>		labelled_statements
%type	<stmt2>		null_statement


%left		','
%right		assignop assignop_ADD assignop_SUB assignop_MOD assignop_AND assignop_MUL assignop_DIV assignop_SHR assignop_SHL assignop_XOR assignop_OR
%right		'?' ':'
%left		logor
%left		logand
%left		'|'
%left		'^'
%left		'&'
%left		eqop neqop
%left		compop_GT compop_GE compop_LT compop_LE
%left		shiftop_R shiftop_L
%left		addop
%left		mulop '*'
%right		unop
%nonassoc	inc dec
%left		'.' pomem

%%



file			: extern_declaration
				{
				startptr = $1;
				$$ = startptr;
				}
			| file extern_declaration
				{
				startptr = (FILTREE *)conectFil($1,$2);
				$$ = startptr;
				yyerrok;
				}
			| error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			| file error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			;

extern_declaration	: func_def
				{
				$$ = $1;
				}
			| declaration
				{
				$$ = (FILTREE *)mkFiltree(fil_DECL,NULL,NULL,$1,NULL,NULL,Linetimes,Columntimes);
				}
			;

func_def		: declarator compound_statement
				{
				$$ = (FILTREE *)mkFiltree(fil_FUNC,NULL,NULL,$1,NULL,$2,Linetimes,Columntimes);
				}
			| declaration_spec declarator compound_statement
				{
				$$ = (FILTREE *)mkFiltree(fil_FUNC,NULL,$1,$2,NULL,$3,Linetimes,Columntimes);
				}
			| declarator declaration_list compound_statement
				{
				$$ = (FILTREE *)mkFiltree(fil_FUNC,NULL,NULL,$1,$2,$3,Linetimes,Columntimes);
				}
			| declaration_spec declarator declaration_list compound_statement
				{
				$$ = (FILTREE *)mkFiltree(fil_FUNC,NULL,$1,$2,$3,$4,Linetimes,Columntimes);
				}
			;

declaration_list	:declaration
				{
				$$ = $1;
				}
			|declaration_list declaration
				{
				$$ = (DCLTREE *)conectDcl($1,$2);
				}
			;
  /***************/
 /* Declaration */
/***************/
declaration		:declaration_spec sc
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_DCLON,NULL,NULL,$1,Linetimes,Columntimes);
				Td_Flag = OFF;
				Td_stnest = 0;
				Td_fnnest = 0;
				}
			|declaration_spec init_declarator_list sc
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_DCLON,NULL,$2,$1,Linetimes,Columntimes);
				Td_Flag = OFF;
				Td_stnest = 0;
				Td_fnnest = 0;
				}
			;

declaration_spec	:storage_class_spec
				{
				$$ = $1;
				}
			|storage_class_spec declaration_spec
				{
				$$ = (SPCTREE *)conectSpc($1,$2);
				}
			|type_spec
				{
				$$ = $1;
				}
			|type_spec declaration_spec
				{
				$$ = (SPCTREE *)conectSpc($1,$2);
				}
			|type_qualifier
				{
				$$ = $1;
				}
			|type_qualifier declaration_spec
				{
				$$ = (SPCTREE *)conectSpc($1,$2);
				}
			;

init_declarator_list	:init_declarator
				{
				$$ = $1;
				}
			|init_declarator_list ',' init_declarator
				{
				$$ = (DCLTREE *)conectDcl($1,$3);
				}
			;

init_declarator		:declarator
				{
				$$ = (DCLTREE *)mkDcldclor(dcl_DCLOR,NULL,$1,NULL,Linetimes,Columntimes);
				if(Td_Flag == ON)
					td_save(Td_Ident);
				}
			|declarator assignop initializer
				{
				$$ = (DCLTREE *)mkDcldclor(dcl_DCLOR,NULL,$1,$3,Linetimes,Columntimes);
				}
			;

storage_class_spec	:AUTO
				{
				$$ = (SPCTREE *)mkSpctree(spc_AUTO,NULL,Linetimes,Columntimes);
				}
			|EXTERN
				{
				$$ = (SPCTREE *)mkSpctree(spc_EXTERN,NULL,Linetimes,Columntimes);
				}
			|REGISTER
				{
				$$ = (SPCTREE *)mkSpctree(spc_REGISTER,NULL,Linetimes,Columntimes);
				}
			|STATIC
				{
				$$ = (SPCTREE *)mkSpctree(spc_STATIC,NULL,Linetimes,Columntimes);
				}
			|TYPEDEF
				{
				$$ = (SPCTREE *)mkSpctree(spc_TYPEDEF,NULL,Linetimes,Columntimes);
				Td_Flag = ON;
				}
			;

type_spec		:CHAR
				{
				$$ = (SPCTREE *)mkSpctree(spc_CHAR,NULL,Linetimes,Columntimes);
				}
			|INT
				{
				$$ = (SPCTREE *)mkSpctree(spc_INT,NULL,Linetimes,Columntimes);
				}
			|SHORT
				{
				$$ = (SPCTREE *)mkSpctree(spc_SHORT,NULL,Linetimes,Columntimes);
				}
			|LONG
				{
				$$ = (SPCTREE *)mkSpctree(spc_LONG,NULL,Linetimes,Columntimes);
				}
			|UNSIGNED
				{
				$$ = (SPCTREE *)mkSpctree(spc_UNSIGNED,NULL,Linetimes,Columntimes);
				}
			|SIGNED
				{
				$$ = (SPCTREE *)mkSpctree(spc_SIGNED,NULL,Linetimes,Columntimes);
				}
			|FLOAT
				{
				$$ = (SPCTREE *)mkSpctree(spc_FLOAT,NULL,Linetimes,Columntimes);
				}
			|DOUBLE
				{
				$$ = (SPCTREE *)mkSpctree(spc_DOUBLE,NULL,Linetimes,Columntimes);
				}
			|VOID
				{
				$$ = (SPCTREE *)mkSpctree(spc_VOID,NULL,Linetimes,Columntimes);
				}
			|struct_type_spec
				{
				$$ = $1;
				}
			|union_type_spec
				{
				$$ = $1;
				}
			|enum_type_spec
				{
				$$ = $1;
				}
			|TYPEDEF_TYPE
				{
				$$ = (SPCTREE *)mkSpctdtype(spc_TD_TYPE,NULL,yylval.symbol,Linetimes,Columntimes);
				}
			;

struct_type_spec	:STRUCT IDENTIFIER '{'
				{
				if(Td_Flag == ON)
					Td_stnest++;
				}
			 member_declaration_list rb
				{
				if(Td_Flag == ON)
					Td_stnest--;
				$$ = (SPCTREE *)mkSpcsue(spc_STRUCT,NULL,$2,$5,Linetimes,Columntimes);
				}
			|STRUCT '{'
				{
				if(Td_Flag == ON)
					Td_stnest++;
				}
			 member_declaration_list rb
				{
				if(Td_Flag == ON)
					Td_stnest--;
				$$ = (SPCTREE *)mkSpcsue(spc_STRUCT,NULL,NULL,$4,Linetimes,Columntimes);
				}
			|STRUCT IDENTIFIER
				{
				$$ = (SPCTREE *)mkSpcsue(spc_STRUCT,NULL,$2,NULL,Linetimes,Columntimes);
				}
			;

union_type_spec		:UNION IDENTIFIER '{'
				{
				if(Td_Flag == ON)
					Td_stnest++;
				}
			 member_declaration_list rb
				{
				if(Td_Flag == ON)
					Td_stnest--;
				$$ = (SPCTREE *)mkSpcsue(spc_UNION,NULL,$2,$5,Linetimes,Columntimes);
				}
			|UNION '{'
				{
				if(Td_Flag == ON)
					Td_stnest++;
				}
			 member_declaration_list rb
				{
				if(Td_Flag == ON)
					Td_stnest--;
				$$ = (SPCTREE *)mkSpcsue(spc_UNION,NULL,NULL,$4,Linetimes,Columntimes);
				}
			|UNION IDENTIFIER
				{
				$$ = (SPCTREE *)mkSpcsue(spc_UNION,NULL,$2,NULL,Linetimes,Columntimes);
				}
			;

member_declaration_list	:member_declaration
				{
				$$ = $1;
				}
			|member_declaration_list member_declaration
				{
				$$ = (DCLTREE *)conectDcl($1,$2);
				}
			;

member_declaration	:spec_qualifier_list member_declarator_list sc
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_MDCLON,NULL,$2,$1,Linetimes,Columntimes);
				}
			;

spec_qualifier_list	:type_spec
				{
				$$ = $1;
				}
			|type_spec spec_qualifier_list
				{
				$$ = (SPCTREE *)conectSpc($1,$2);
				}
			|type_qualifier
				{
				$$ = $1;
				}
			|type_qualifier spec_qualifier_list
				{
				$$ = (SPCTREE *)conectSpc($1,$2);
				}
			;

member_declarator_list	:member_declarator
				{
				$$ = $1;
				}
			|member_declarator_list ',' member_declarator
				{
				$$ = (DCLTREE *)conectDcl($1,$3);
				}
			;

member_declarator	:declarator
				{
				$$ = (DCLTREE *)mkDcldclor(dcl_MDCLOR,NULL,$1,NULL,Linetimes,Columntimes);
				}
			|':' initializer
				{
				$$ = (DCLTREE *)mkDcldclor(dcl_MDCLOR,NULL,NULL,$2,Linetimes,Columntimes);
				}
			|declarator ':' initializer
				{
				$$ = (DCLTREE *)mkDcldclor(dcl_MDCLOR,NULL,$1,$3,Linetimes,Columntimes);
				}
			;

enum_type_spec		:ENUM IDENTIFIER '{' enumerator_list rb
				{
				$$ = (SPCTREE *)mkSpcsue(spc_ENUM,NULL,$2,$4,Linetimes,Columntimes);
				}
			|ENUM IDENTIFIER
				{
				$$ = (SPCTREE *)mkSpcsue(spc_ENUM,NULL,$2,NULL,Linetimes,Columntimes);
				}
			|ENUM  '{' enumerator_list rb
				{
				$$ = (SPCTREE *)mkSpcsue(spc_ENUM,NULL,NULL,$3,Linetimes,Columntimes);
				}
			;

enumerator_list		:enumerator
				{
				$$ = $1;
				}
			|enumerator_list ',' enumerator
				{
				$$ = (DCLTREE *)conectDcl($1,$3);
				}
			;

enumerator		:IDENTIFIER
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_IDENT,NULL,$1,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcldclor(dcl_EDCLOR,NULL,p,NULL,Linetimes,Columntimes);
				}
			|IDENTIFIER assignop initializer
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_IDENT,NULL,$1,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcldclor(dcl_EDCLOR,NULL,p,$3,Linetimes,Columntimes);
				}
			;

type_qualifier		:CONST
				{
				$$ = (SPCTREE *)mkSpctree(spc_CONST,NULL,Linetimes,Columntimes);
				}
			|VOLATILE
				{
				$$ = (SPCTREE *)mkSpctree(spc_VOLATILE,NULL,Linetimes,Columntimes);
				}
			;

declarator		: pointer declarator2
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_PTR,NULL,$1,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($2,p);
				}
			| declarator2
				{
				$$ = $1;
				}
			;

declarator2		: IDENTIFIER
				{
				$$ = (DCLTREE *)mkDcltree(dcl_IDENT,NULL,$1,Linetimes,Columntimes);
				if(Td_Flag == ON && Td_stnest == 0 && Td_fnnest == 0)
					strcpy(Td_Ident,$1);
				}
			| '(' declarator rp
				{
				$$ = $2;
				}
			| declarator2 '[' ']'
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_ARRAY,NULL,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			| declarator2 '[' nc_expression ']'
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_ARRAY,NULL,$3,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			| declarator2 '(' rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			| declarator2 '(' param_type_list rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,$3,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			| declarator2 '(' error rp
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			| declarator2 '(' ident_list rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,$3,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			;

pointer			:'*'
				{
				$$ = (SPCTREE *)mkSpctree(spc_POINTER,NULL,Linetimes,Columntimes);
				}
			|'*' type_qualifier_list
				{
				$$ = (SPCTREE *)mkSpctree(spc_POINTER,$2,Linetimes,Columntimes);
				}
			|'*' pointer
				{
				$$ = (SPCTREE *)mkSpctree(spc_POINTER,$2,Linetimes,Columntimes);
				}
			|'*' type_qualifier_list pointer
				{
				SPCTREE *p;
				p = (SPCTREE *)conectSpc($2,$3);
				$$ = (SPCTREE *)mkSpctree(spc_POINTER,p,Linetimes,Columntimes);
				}
			;

type_qualifier_list	:type_qualifier
				{
				$$ = $1;
				}
			|type_qualifier_list type_qualifier
				{
				$$ = (SPCTREE *)conectSpc($1,$2);
				}
			;

ident_list		:IDENTIFIER
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_IDENT,NULL,$1,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcldclon(dcl_PDCLON,NULL,p,NULL,Linetimes,Columntimes);
				}
			|ident_list ',' IDENTIFIER
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_IDENT,NULL,$3,Linetimes,Columntimes);
				p = (DCLTREE *)mkDcldclon(dcl_PDCLON,NULL,p,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			;

param_type_list		:param_list
				{
				$$ = $1;
				}
			|param_list ',' DDDOT
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_DDDOT,NULL,NULL,Linetimes,Columntimes);
				p = (DCLTREE *)mkDcldclon(dcl_PDCLON,NULL,p,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			;

param_list		: param_decl
				{
				$$ = $1;
				}
			| param_list ',' param_decl
				{
				$$ = (DCLTREE *)conectDcl($1,$3);
				yyerrok;
				}
			| param_list error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			| param_list error param_decl
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				yyerrok;
				exit(0);
				}
			| param_list ',' error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			;

param_decl		: declaration_spec declarator
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_PDCLON,NULL,$2,$1,Linetimes,Columntimes);
				}
			| declaration_spec
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_PDCLON,NULL,NULL,$1,Linetimes,Columntimes);
				}
			| declaration_spec abst_declarator
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_PDCLON,NULL,$2,$1,Linetimes,Columntimes);
				}
			;


type_name		: spec_qualifier_list
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_DCLON,NULL,NULL,$1,Linetimes,Columntimes);
				}
			| spec_qualifier_list abst_declarator
				{
				$$ = (DCLTREE *)mkDcldclon(dcl_DCLON,NULL,$2,$1,Linetimes,Columntimes);
				}
			;

abst_declarator		: pointer
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_PTR,NULL,$1,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcltree(dcl_NONE,p,NULL,Linetimes,Columntimes);
				}
			| pointer abst_declarator2
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_PTR,NULL,$1,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($2,p);
				}
			| abst_declarator2
				{
				$$ = $1;
				}
			;

abst_declarator2	: '(' abst_declarator rp
				{
				$$ = $2;
				}
			|'[' ']'
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_ARRAY,NULL,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcltree(dcl_NONE,p,NULL,Linetimes,Columntimes);
				}
			|'[' nc_expression ']'
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_ARRAY,NULL,$2,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcltree(dcl_NONE,p,NULL,Linetimes,Columntimes);
				}
			|abst_declarator2 '[' ']'
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_ARRAY,NULL,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			|abst_declarator2 '['nc_expression ']'
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_ARRAY,NULL,$3,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			|'(' rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,NULL,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcltree(dcl_NONE,p,NULL,Linetimes,Columntimes);
				}
			|'(' param_type_list rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,$2,Linetimes,Columntimes);
				$$ = (DCLTREE *)mkDcltree(dcl_NONE,p,NULL,Linetimes,Columntimes);
				}
			|abst_declarator2 '(' rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,NULL);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			|abst_declarator2 '(' param_type_list rp
				{
				DCLTREE *p;
				p = (DCLTREE *)mkDcltree(dcl_FUNC,NULL,$3,Linetimes,Columntimes);
				$$ = (DCLTREE *)conectDcl($1,p);
				}
			;

initializer		:nc_expression
				{
				$$ = (DCLTREE *)mkInit(dcl_INIT,NULL,NULL,$1,Linetimes,Columntimes);
				}
			|'{' initializer_list rb
				{
				$$ = (DCLTREE *)mkInit(dcl_INIT,NULL,$2,NULL,Linetimes,Columntimes);
				}
			|'{' initializer_list ',' rb
				{
				$$ = (DCLTREE *)mkInit(dcl_INIT,NULL,$2,NULL,Linetimes,Columntimes);
				}
			;

initializer_list	:initializer
				{
				$$ = $1;
				}
			|initializer_list ',' initializer
				{
				$$ = (DCLTREE *)conectDcl($1,$3);
				}
			;
  /*************/
 /* Statement */
/*************/
statement_list		:/* empty */
				{
				$$ = makestmtree1(EMPTY_STK,NULL,NULL,Linetimes,Columntimes);
				}
			|statement_list statement
				{
				$$ = (STMTREE1 *)conectSTM($1,$2);
				yyerrok;
				}
			|statement_list error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
                        ;
statement		:simple_statement
				{
				$$ = makestmtree1(SIMPLE_STK,NULL,$1,Linetimes,Columntimes);
				}
			|compound_statement
				{
				$$ = makestmtree1(COMPOU_STK,NULL,$1,Linetimes,Columntimes);
				}
			|conditional_statements
				{
				$$ = makestmtree1(CONDIT_STK,NULL,$1,Linetimes,Columntimes);
				}
			|iteration_statements
				{
				$$ = makestmtree1(ITERAT_STK,NULL,$1,Linetimes,Columntimes);
				}
			|selection_statements
				{
				$$ = makestmtree1(SELECT_STK,NULL,$1,Linetimes,Columntimes);
				}
			|jump_statements
				{
				$$ = makestmtree1(JUMP_STK,NULL,$1,Linetimes,Columntimes);
				}
			|labelled_statements
				{
				$$ = makestmtree1(LABELL_STK,NULL,$1,Linetimes,Columntimes);
				}
			|null_statement
				{
				$$ = makestmtree1(NULL_STK,NULL,$1,Linetimes,Columntimes);
				}
			|selection_statements1
				{
				$$ = makestmtree1(SELECT1_STK,NULL,$1,Linetimes,Columntimes);
				}
			;
simple_statement	:expression sc
				{
				$$ = makestmtree2(SIMPLE_STC,NULL,NULL,NULL,NULL,NULL,$1,NULL,NULL,Linetimes,Columntimes);
				}
			;
compound_statement	:'{' rb
				{
				$$ = makestmtree2(COMPOU_STC,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			| '{' declaration_list rb
				{
				$$ = makestmtree2(COMPOU_STC,NULL,NULL,NULL,NULL,NULL,NULL,NULL,$2,Linetimes,Columntimes);
				}
			| '{' statement_list rb
				{
				$$ = makestmtree2(COMPOU_STC,NULL,NULL,NULL,$2,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			| '{' declaration_list statement_list rb
				{
				$$ = makestmtree2(COMPOU_STC,NULL,NULL,NULL,$3,NULL,NULL,NULL,$2,Linetimes,Columntimes);
				}
			;

conditional_statements	:IF'(' expression rp statement
				{
				$$ = makestmtree2(IF_STC,$3,NULL,NULL,$5,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|IF'('expression rp statement ELSE statement
				{
				$$ = makestmtree2(IF_ELSE_STC,$3,NULL,NULL,$5,$7,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			;

iteration_statements 	:WHILE'('expression rp statement
				{
				$$ = makestmtree2(WHILE_STC,$3,NULL,NULL,$5,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|DO statement WHILE'('expression rp sc
				{
				$$ = makestmtree2(DO_WHILE_STC,$5,NULL,NULL,$2,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|FOR '(' sc sc rp statement
				{
				$$ = makestmtree2(FOR_STC,NULL,NULL,NULL,$6,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|FOR '('expression sc sc rp statement
				{
				$$ = makestmtree2(FOR_STC,$3,NULL,NULL,$7,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|FOR '('expression sc  expression sc rp statement
				{
				$$ = makestmtree2(FOR_STC,$3,$5,NULL,$8,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|FOR '('expression sc  expression sc expression rp statement
				{
				$$ = makestmtree2(FOR_STC,$3,$5,$7,$9,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
		 	|FOR '(' sc expression  sc expression rp statement
				{
				$$ = makestmtree2(FOR_STC,NULL,$4,$6,$8,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
        		|FOR '('sc expression sc rp statement
				{
				$$ = makestmtree2(FOR_STC,NULL,$4,NULL,$7,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|FOR '('expression sc sc expression rp statement
				{
				$$ = makestmtree2(FOR_STC,$3,NULL,$6,$8,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|FOR '('sc sc expression rp statement
				{
				$$ = makestmtree2(FOR_STC,NULL,NULL,$5,$7,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			;
selection_statements	:SWITCH '(' expression rp statement
				{
				$$ = makestmtree2(SWITCH_STC,$3,NULL,NULL,$5,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			;
selection_statements1	:CASE nc_expression ':'statement
				{
				$$ = makestmtree2(CASE_STC,$2,NULL,NULL,$4,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			|DEFAULT ':'statement
				{
				$$ = makestmtree2(DEFAULT_STC,NULL,NULL,NULL,$3,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			;
jump_statements		: BREAK sc
				{
				$$ = makestmtree2(BREAK_STC,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			| CONTINUE sc
				{
				$$ = makestmtree2(CONTINUE_STC,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			| RETURN sc
				{
				$$ = makestmtree2(RETURN_STC,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			| RETURN expression sc
				{
				$$ = makestmtree2(RETURN_STC,$2,NULL,NULL,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
			| GOTO IDENTIFIER sc
				{
				$$ = makestmtree2(GOTO_STC,NULL,NULL,NULL,NULL,NULL,NULL,$2,NULL,Linetimes,Columntimes);
				}
			;
labelled_statements	:IDENTIFIER ':' statement
				{
				$$ = makestmtree2(LABBEL_STC,NULL,NULL,NULL,$3,NULL,NULL,$1,NULL,Linetimes,Columntimes);
				}
			;
null_statement		:sc
				{
				$$ = makestmtree2(STMT_NULL_STC,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,Linetimes,Columntimes);
				}
                	;

  /**************/
 /* Expression */
/**************/
nc_expression		:linear_operators
				{
				$$ = (EXPR *)makeline(op_LINER, NULL, $1,Linetimes,Columntimes);
				}
			|'*' nc_expression %prec unop
				{
				$$ = (EXPR *)makenode1(op_INDIR, NULL, $2,Linetimes,Columntimes);
				}
			|'&' nc_expression %prec unop
				{
				$$ = (EXPR *)makenode1(op_ADR, NULL, $2,Linetimes,Columntimes);
				}
			|addop nc_expression %prec unop
				{
				$$ = (EXPR *)makenode1($1 == '+' ? op_PLUS : op_MINUS, NULL, $2,Linetimes,Columntimes);
				}
			|unop nc_expression
				{
				$$ = (EXPR *)makenode1($1 == '!' ? op_LNOT : op_BNOT, NULL, $2,Linetimes,Columntimes);
				}
			|inc nc_expression
				{
				$$ = (EXPR *)makenode1(op_PREINC, NULL, $2,Linetimes,Columntimes);
				}
			|dec nc_expression
				{
				$$ = (EXPR *)makenode1(op_PREDEC, NULL, $2,Linetimes,Columntimes);
				}
			|nc_expression inc
				{
				$$ = (EXPR *)makenode1(op_POSTINC, NULL,$1,Linetimes,Columntimes);
				}
			|nc_expression dec
				{
				$$ = (EXPR *)makenode1(op_POSTDEC, NULL,$1,Linetimes,Columntimes);
				}
			|'(' type_name rp nc_expression %prec unop
				{
				$$ = (EXPR *)makecast(op_CAST, NULL, $2, $4,Linetimes,Columntimes);
				}
			|SIZEOF nc_expression %prec unop
				{
				$$ = (EXPR *)makenodeS(op_SIZEOF_I, NULL, $2,Linetimes,Columntimes);
				}
			|SIZEOF '(' type_name rp %prec unop
				{
				$$ = (EXPR *)makenodeS(op_SIZEOF_D, NULL, $3,Linetimes,Columntimes);
				}
            	    	|nc_expression '*' nc_expression
				{
				$$ = (EXPR *)makenode2(op_MUL, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression mulop nc_expression
				{
				$$ = (EXPR *)makenode2($2 == '/' ? op_DIV : op_MOD, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression addop nc_expression
				{
				$$ = (EXPR *)makenode2($2 == '+' ? op_ADD : op_SUB, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression shiftop_R nc_expression
				{
				$$ = (EXPR *)makenode2(op_SHR, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression shiftop_L nc_expression
				{
				$$ = (EXPR *)makenode2(op_SHL, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression compop_GT nc_expression
				{
				$$ = (EXPR *)makenode2(op_GT, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression compop_GE nc_expression
				{
				$$ = (EXPR *)makenode2(op_GE, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression compop_LT nc_expression
				{
				$$ = (EXPR *)makenode2(op_LT, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression compop_LE nc_expression
				{
				$$ = (EXPR *)makenode2(op_LE, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression eqop nc_expression
				{
				$$ = (EXPR *)makenode2(op_EQ, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression neqop nc_expression
				{
				$$ = (EXPR *)makenode2(op_NEQ, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression '&' nc_expression
				{
				$$ = (EXPR *)makenode2(op_AND, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression '^' nc_expression
				{
				$$ = (EXPR *)makenode2(op_XOR, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression '|' nc_expression
				{
				$$ = (EXPR *)makenode2(op_OR, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression logand nc_expression
				{
				$$ = (EXPR *)makenode2(op_LAND, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression logor nc_expression
				{
				$$ = (EXPR *)makenode2(op_LOR, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression '?' nc_expression ':' nc_expression
				{
				$$ = (EXPR *)makenode3(op_COND, NULL, $1, $3, $5,Linetimes,Columntimes);
				}
			|nc_expression assignop nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_ADD nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_ADD, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_SUB nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_SUB, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_MOD nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_MOD, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_AND nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_AND, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_MUL nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_MUL, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_DIV nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_DIV, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_SHR nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_SHR, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_SHL nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_SHL, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_XOR nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_XOR, NULL, $1, $3,Linetimes,Columntimes);
				}
			|nc_expression assignop_OR nc_expression
				{
				$$ = (EXPR *)makenode2(op_ASSIGN_OR, NULL, $1, $3,Linetimes,Columntimes);
				}
			;
expression		:nc_expression
				{
				$$ = $1;
				}
			|expression ',' nc_expression
				{
				$$ = (EXPR *)conectExp($1, $3);
				yyerrok;
				}
			|expression error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			|expression error nc_expression
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			|expression ',' error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			;

linear_operators	:IDENTIFIER
				{
				$$ = (LINER *)makelineid(li_IDENT, NULL, $1),Linetimes,Columntimes;
				}
			|CONSTANT
				{
				$$ = (LINER *)makeconst(li_CONST, NULL, $1.va,$1.sp,Linetimes,Columntimes);
				}
			|STRINGS
				{
				$$ = (LINER *)makelineid(li_STR, NULL, $1,Linetimes,Columntimes);
				}
			|CHARACTER
				{
				$$ = (LINER *)makelineid(li_CHARA, NULL, $1,Linetimes,Columntimes);
				}
			|'(' nc_expression rp
				{
				$$ = (LINER *)makelinexp(li_SAME, NULL, $2,Linetimes,Columntimes);
				}
			|linear_operators  '[' expression ']'
				{
				$$ = (LINER *)makefa(li_ARRAY, NULL, $1, $3,Linetimes,Columntimes);
				}
			|linear_operators '(' rp
				{
				$$ = (LINER *)makefa(li_FUNC, NULL, $1, NULL,Linetimes,Columntimes);
				}
			|linear_operators '(' expression_list rp
				{
				$$ = (LINER *)makefa(li_FUNC, NULL, $1, $3,Linetimes,Columntimes);
				}
			|linear_operators '.' IDENTIFIER
				{
				$$ = (LINER *)makepost(li_COMMA, NULL, $1, $3,Linetimes,Columntimes);
				}
			|linear_operators pomem IDENTIFIER
				{
				$$ = (LINER *)makepost(li_POMEM, NULL, $1, $3,Linetimes,Columntimes);
				}
			;

expression_list		:nc_expression
				{
				$$ = $1;
				}
			|expression_list ',' nc_expression
				{
				$$ = (EXPR *)conectExp($1,$3);
				yyerrok;
				}
			|error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			|expression_list error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			|expression_list error nc_expression
 				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				yyerrok;
				exit(0);
				}
			|expression_list ',' error
				{
				printf("Syntax Error:Please Check your source file.\n\tusage:%>cc -c [filename].\n");
				exit(0);
				}
			;


rb			: '}' {yyerrok;}
			;
rp			: ')' {yyerrok;}
			;
sc			: ';' {yyerrok;}
			;

%%
